import os
import sys
import json
import argparse

from tinyddp.server import DDPServer
from tinyddp.dispatch import DDPDispatcher

import daemon
import common
import logger
from sg_config import Configer
from link import LinkManager
from sequence import Sequencer
from plugin import PluginManager
from systeminfo import SystemInfo
from action import Action

def prepare_run():
    path = "..{}plugins".format(os.sep)
    if not os.path.exists(path):
        os.makedirs(path)

    folder_list = ['config']
    for floder in folder_list:
        path = '.{}{}'.format(os.sep, floder)
        if not os.path.exists(path):
            os.makedirs(path)

            if path == '.{}config'.format(os.sep):
                content = {
                    "enable_console" : "True",
                    "_log_level_option" : ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"],
                    "log_level" : "NOTSET",
                    "logger_path" : "./log",
                    "old_file_elapsed_day": 5,
                    "mongodb": "mongodb://127.0.0.1:37432/"
                }
                core_conf_file = '.{}config{}core_config.json'.format(os.sep, os.sep)
                with open(core_conf_file, 'w', encoding='utf-8') as file_handle:
                    json.dump(content, file_handle, ensure_ascii=False, indent=4)

def main():
    if common.is_linux():
        dprocess = daemon.Daemon("archon.core")
        dprocess.start()

    # decode argument
    parser = argparse.ArgumentParser()
    parser.add_argument('--port', help='Arcon port', type=int, default=38600)

    args = parser.parse_args()
    port = args.port

    prepare_run()

    dispatcher = DDPDispatcher()
    plugin_manager = PluginManager()

    del sys.argv[1:]
    server = DDPServer("0.0.0.0", port, dispatcher)
    configer = Configer()
    sequencer = Sequencer(dispatcher)
    system = SystemInfo()
    link_mgr = LinkManager()
    action = Action()

    dispatcher.register_instance(action, "action.")
    dispatcher.register_instance(sequencer, "sequence.")
    dispatcher.register_instance(configer, "config.")
    dispatcher.register_instance(system, "systeminfo.")
    dispatcher.register_instance(plugin_manager, "plugin.")
    dispatcher.register_instance(link_mgr, "linkmgr.")

    logger.log().info("Arcon start, port is {}!".format(port))

    server.serve_forever()

if __name__ == "__main__":
    main()
