# Model
* None


## Description
Archon system interface.


## Vendor
* None


## exec(command)
### Function Description
Execute system command or load program.
### Parameters
* command: str, system command or program command.
### Returns
* None
### Examples
```
exec('ls /root -l')
```


## scp(command)
### Function Description
Secure copy.
### Parameters
* command: str, scp [-r] [-P port] [[user@]host1:]file1 ... [[user@]host2:]file2 password.
### Returns
* None
### Examples
```
scp('-r -P 22 root@169.254.1.32:/root/test D:\')
```


## calc(algo)
### Function Description
Calculation.
### Parameters
* algo: algorithm formula.
### Returns
* int/float, caculate result.
### Examples
```
print(calc(1+1))
```


## input(msg)
### Function Description
Input value from input box.
### Parameters
* msg: str, input box tips message.
### Returns
* Any, value from input box.
### Examples
```
print(input('please input SN'))
```


## output(msg)
### Function Description
Output message to output box.
### Parameters
* msg: str, output box tips message.
### Returns
* None
### Examples
```
output('Please select channel one and click the confirm button!')
```

## combobox_input(msg, items)
### Function Description
Input value from input combobox.
### Parameters
* msg: str, input combobox tips message.
* items: list, input combobox select list.
### Returns
* Any, value from input combobox.
### Examples
```
print(combobox_input('please select bank35 voltage', [1.8, 2.5, 3.3]))
```

## confirm_output(msg)
### Function Description
Output message to output box with confirm button.
### Parameters
* msg: str, output box tips message.
### Returns
* None
### Examples
```
confirm_output('Please select channel one and click the confirm button!')
```


## confirm_stop_output(msg)
### Function Description
Output message to output box with confirm and stop button.
### Parameters
* msg: str, output box tips message.
### Returns
* None
### Examples
```
confirm_stop_output('Please select channel one and click the confirm button or click the stop button to stop test!')
```


## alert_output(msg)
### Function Description
Output alert message to output box.
### Parameters
* msg: str, output box tips message.
### Returns
* None
### Examples
```
alert_output('WARNING: The temperature reaches the threshold!')
```

## error_output(msg)
### Function Description
Output error message to output box.
### Parameters
* msg: str, output box tips message.
### Returns
* None
### Examples
```
error_output('EEROR: The temperature exceeds the threshold!')
```


## scaner()
### Function Description
Scaner barcode.
### Parameters
* length: int, barcode length.
### Returns
* str, barcode.
### Examples
```
print(scaner())
```


## set_sn()
### Function Description
Set SN.
### Parameters
* serial_number: str, serial number.
### Returns
* str, serial number.
### Examples
```
print(set_sn("GQQ02950500Q17584"))
```


## ping(command)
### Function Description
Pings a remote host and handles the responses.
### Parameters
* command: str, The remote hostname or IP address to ping, ping [options] <destination>.
### Returns
* str
### Examples
```
print(ping('-c 1 -W 0.1 169.254.1.32'))
```


## if(condition)
### Function Description
The scoped start identifier of the conditional statement block, paired with if_end.
### Parameters
* condition: The condition of judgment
### Returns
* None
### Examples
* None


## elif(condition)
### Function Description
An if statement can be followed by zero or more else if statements, which must precede the else statement.
### Parameters
* condition: The condition of judgment
### Returns
* None
### Examples
* None


## else()
### Function Description
An if statement can be followed by zero or one else, but if there is an else if statement, the else statement needs to be followed by the else if statement.
### Parameters
* None
### Returns
* None
### Examples
* None


## if_end()
### Function Description
The end of scope identifier for a conditional statement block, paired with if
### Parameters
* None
### Returns
* None
### Examples
* None


## assert(condition)
### Function Description
Used to determine an expression. The test case result is fail when the expression condition is false.
### Parameters
* condition: The condition of judgment
### Returns
* None
### Examples
* None


## loop(count)
### Function Description
Identifier at the beginning of the loop scope, paired with loop end
### Parameters
* count: int, "int" for infinite loop
### Returns
* None
### Examples
* None


## loop_end()
### Function Description
Identifier at the end of the loop scope, used in pairs with loop
### Parameters
* None
### Returns
* None
### Examples
* None


## break()
### Function Description
Used in the loop scope to exit the current loop.
### Parameters
* None
### Returns
* None
### Examples
* None


## var(command)
### Function Description
Variables can be defined in use cases.
### Parameters
* value: The value of the variable
### Returns
* None
### Examples
```
var(123)
```


## timeout(timeout_ms)
### Function Description
Set the timeout for test case. The timeout is global, which is set for all cases until the time is reset.
### Parameters
* timeout_ms: int, unit ms.
### Returns
* None
### Examples
```
timeout(3000)
```


## delay(time_ms)
### Function Description
delay
### Parameters
* time_ms: int, unit ms.
### Returns
* None
### Examples
```
delay(1000) # delay 1 second.
```


## run_app(file)
### Function Description
Run executable program.
### Parameters
* file: str, the full path of program.
### Returns
* None
### Examples
```
run_app('./test.exe')
```


## load_library(path)
### Function Description
Load libraries, including C libraries and Python libraries
### Parameters
* path: str, the full path of library.
### Returns
* None
### Examples
```
load_library('./libtest.so')
```
