import re
import socket
import select


class TCPDef:
    RECV_LEN = 2048


class _TCPMessage():
    """TCPMessage is support for reply message to TCP Client.

    Args:
        socket (object): TCP Server accept socket.
        CRLF (str): TCP Message endle, also you can reserve blank and
                    send command with endle every time.

    Attributes:
        ip (str): TCP accept ip.
        port (int): TCP accept port.

    Examples:
        >>> tcp = TCPServer(setting='0.0.0.0:7600', CRLF='\\r\\n')
        message = tcp.accept("169.254.1.99")
    """

    def __init__(self, socket: object, CRLF="") -> None:
        pass

    def send(self, command: str) -> None:
        """Send command to TCP client.

        Args:
            command (str): command to TCP client.

        Examples:
            >>> tcp = TCPServer(setting='0.0.0.0:7600', CRLF='\\r\\n')
            message = tcp.accept("169.254.1.99")
            message.send('message')
        """

    def recv(self) -> str:
        """Recive reply message from TCP client.

        Returns:
            str: Reply data from client.

        Examples:
            >>> tcp = TCPServer(setting='0.0.0.0:7600', CRLF='\\r\\n')
            message = tcp.accept("169.254.1.99")
            message.send('message')
            print(message.recv())
        """

    def query(self, command: str) -> str:
        """Recive reply message from TCP client.

        Args:
            command (str): command to TCP client.

        Returns:
            str: Reply data from client.

        Examples:
            >>> tcp = TCPServer(setting='0.0.0.0:7600', CRLF='\\r\\n')
            message = tcp.accept("169.254.1.99")
            print(message.query('message'))
        """
