# Model
* None


## Description
Support Serial protocol.


## Vendor
* None

## Class Name
* Serial
### Parameters
* partno: str, Device PartNo.
* setting: str, Serial connect setting. Format is "port, baudrate, bytesize, parity, stopbits".
* CRLF: str, Serial Message endle, also you can reserve blank and send command with endle every time.
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
```

## write(command)
### Function Description
Send message via uart bus.
### Parameters
* command: str, command to uart device.
### Returns
* None
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
uart.write('hello world')
```

## write_hex(command)
### Function Description
Send hex message via uart bus.
### Parameters
* command: list, hex command to uart device.
### Returns
* None
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
uart.write_hex([10,20,30])
```

## read(len)
### Function Description
Recive reply via uart.
### Parameters
* len: int, ength of recive data.
### Returns
* str
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
uart.write('hello world')
print(uart.read())
```

## read_hex(len)
### Function Description
Recive reply via uart, and return with ord list.
### Parameters
* len: int, ength of recive data.
### Returns
* list
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
uart.write_hex([10,20,30])
print(uart.read_hex())
```

## read_bytes(len)
### Function Description
Recive reply via uart, and return bytes data.
### Parameters
* len: int, ength of recive data.
### Returns
* bytes
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
uart.write_hex([10,20,30])
print(uart.read_bytes())
```

## query(command)
### Function Description
Send command to via uart, then recive reply data.
### Parameters
* command: str, command to uart device.
### Returns
* str
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
print(uart.query('hello world'))
```

## sendfile_ymodem(file)
### Function Description
sendfile with ymodem protocol.
### Parameters
* file: str, file fullpath.
### Returns
* bool, True is successed, False is failed.
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
print(uart.sendfile_ymodem('d:\\test.bin'))
```

## sendfile_xmodem(file)
### Function Description
sendfile with xmodem protocol.
### Parameters
* file: str, file fullpath.
### Returns
* bool, True is successed, False is failed.
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
print(uart.sendfile_xmodem('d:\\test.bin'))
```

## sendfile_zmodem(file)
### Function Description
sendfile with zmodem protocol.
### Parameters
* file: str, file fullpath.
### Returns
* bool, True is successed, False is failed.
### Examples
```
uart = Serial(partno="IT8812C", setting="COM1,115200,8,none,1", CRLF='\r\n')
print(uart.sendfile_zmodem('d:\\test.bin'))
```
